from roofai.help.google_maps.geocode import help_geocode
from roofai.help.google_maps.predict import help_predict
from roofai.help.google_maps.static_api import help_get_static_image

help_functions = {
    "geocode": help_geocode,
    "get_static_image": help_get_static_image,
    "predict": help_predict,
}
